@@ -8,14 +8,22 @@
 
 (ns ^{:doc "A caching library for Clojure."
       :author "Fogus"}
-  clojure.core.cache)
+  clojure.core.cache
+  (:require clojure.data.priority-map)
+  (:import (java.lang.ref ReferenceQueue SoftReference)
+           (java.util.concurrent ConcurrentHashMap)))
+
+(set! *warn-on-reflection* true)
 
 ;; # Protocols and Types
 
 (defprotocol CacheProtocol
   "This is the protocol describing the basic cache capability."
-  (lookup  [cache e]
-   "Retrieve the value associated with `e` if it exists")
+  (lookup [cache e]
+          [cache e not-found]
+   "Retrieve the value associated with `e` if it exists, else `nil` in
+   the 2-arg case.  Retrieve the value associated with `e` if it exists,
+   else `not-found` in the 3-arg case.")
   (has?    [cache e]
    "Checks if the cache contains a value associtaed with `e`")
   (hit     [cache e]
@@ -31,13 +39,27 @@
    The contract is that said cache should return an instance of its
    own type."))
 
+(def ^{:private true} default-wrapper-fn #(%1 %2))
+
+(defn through
+  "The basic hit/miss logic for the cache system.  Expects a wrap function and
+  value function.  The wrap function takes the value function and the item in question
+  and is expected to run the value function with the item whenever a cache
+  miss occurs.  The intent is to hide any cache-specific cells from leaking
+  into the cache logic itelf."
+  ([cache item] (through default-wrapper-fn identity cache item))
+  ([value-fn cache item] (through default-wrapper-fn value-fn cache item))
+  ([wrap-fn value-fn cache item]
+    (if (clojure.core.cache/has? cache item)
+      (clojure.core.cache/hit cache item)
+      (clojure.core.cache/miss cache item (wrap-fn #(value-fn %) item)))))
 
 (defmacro defcache
   [type-name fields & specifics]
   (let [[base-field & _] fields]
     `(deftype ~type-name [~@fields]
        ~@specifics
-     
+
        clojure.lang.ILookup
        (valAt [this# key#]
               (lookup this# key#))
@@ -69,21 +91,41 @@
        (empty [this#]
          (seed this# (empty ~base-field)))
        (equiv [_# other#]
-         (.equiv ~base-field other#))
+         (clojure.lang.Util/equiv ~base-field other#))
 
        clojure.lang.Seqable
        (seq [_#]
-         (seq ~base-field))
-
-       ;; Java interfaces
-       java.lang.Iterable
-       (iterator [this#] (.iterator ~base-field)))))
-
+         (seq ~base-field)))))
 
 (defcache BasicCache [cache]
   CacheProtocol
   (lookup [_ item]
     (get cache item))
+  (lookup [_ item not-found]
+    (get cache item not-found))
+  (has? [_ item]
+    (contains? cache item))
+  (hit [this item] this)
+  (miss [_ item result]
+    (BasicCache. (assoc cache item result)))
+  (evict [_ key]
+    (BasicCache. (dissoc cache key)))
+  (seed [_ base]
+    (BasicCache. base))
+  Object
+  (toString [_] (str cache)))
+
+;; FnCache
+
+(defcache FnCache [cache f]
+  CacheProtocol
+  (lookup [_ item]
+    (f (get cache item)))
+  (lookup [_ item not-found]
+    (let [ret (get cache item not-found)]
+      (if (= ret not-found)
+        not-found
+        (f ret))))
   (has? [_ item]
     (contains? cache item))
   (hit [this item] this)
@@ -99,7 +141,7 @@
 ;; # FIFO
 
 (defn- describe-layout [mappy limit]
-  (let [q clojure.lang.PersistentQueue/EMPTY 
+  (let [q clojure.lang.PersistentQueue/EMPTY
         ks (keys mappy)
         [dropping keeping] (split-at (- (count ks) limit) ks)]
     {:dropping dropping
@@ -121,14 +163,19 @@
   CacheProtocol
   (lookup [_ item]
     (get cache item))
+  (lookup [_ item not-found]
+    (get cache item not-found))
   (has? [_ item]
     (contains? cache item))
   (hit [this item]
     this)
   (miss [_ item result]
-    (let [k (peek q)]
-      (FIFOCache. (-> cache (dissoc k) (assoc item result))
-                  (-> q pop (conj item))
+    (let [[kache qq] (let [k (peek q)]
+                       (if (>= (count cache) limit)
+                         [(dissoc cache k) (pop q)]
+                         [cache (pop q)]))]
+      (FIFOCache. (assoc kache item result)
+                  (conj qq item)
                   limit)))
   (evict [this key]
     (let [v (get cache key ::miss)]
@@ -147,11 +194,19 @@
   (toString [_]
     (str cache \, \space (pr-str q))))
 
+(defn- build-leastness-queue
+  [base limit start-at]
+  (into (clojure.data.priority-map/priority-map)
+        (concat (take (- limit (count base)) (for [k (range (- limit) 0)] [k k]))
+                (for [[k _] base] [k start-at]))))
+
 
 (defcache LRUCache [cache lru tick limit]
   CacheProtocol
   (lookup [_ item]
     (get cache item))
+  (lookup [_ item not-found]
+    (get cache item not-found))
   (has? [_ item]
     (contains? cache item))
   (hit [_ item]
@@ -162,12 +217,13 @@
                  limit)))
   (miss [_ item result]
     (let [tick+ (inc tick)]
-      (if-let [ks (keys lru)]
-        (let [k (apply min-key lru ks)]
-          (LRUCache. (-> cache (dissoc k) (assoc item result))  ;; expulsion case
-                     (-> lru (dissoc k) (assoc item tick+))
-                     tick+
-                     limit))
+      (if (>= (count lru) limit)
+        (let [k (if (contains? lru item)
+                  item
+                  (first (peek lru))) ;; minimum-key, maybe evict case
+              c (-> cache (dissoc k) (assoc item result))
+              l (-> lru (dissoc k) (assoc item tick+))]
+          (LRUCache. c l tick+ limit))
         (LRUCache. (assoc cache item result)  ;; no change case
                    (assoc lru item tick+)
                    tick+
@@ -178,70 +234,77 @@
         this
         (LRUCache. (dissoc cache key)
                    (dissoc lru key)
-                   tick
+                   (inc tick)
                    limit))))
   (seed [_ base]
     (LRUCache. base
-               (into {} (for [x (range (- limit) 0)] [x x]))
+               (build-leastness-queue base limit 0)
                0
                limit))
   Object
   (toString [_]
     (str cache \, \space lru \, \space tick \, \space limit)))
 
-(declare key-killer)
 
-(defcache TTLCache [cache ttl limit]
+(defn- key-killer
+  [ttl expiry now]
+  (let [ks (map key (filter #(> (- now (val %)) expiry) ttl))]
+    #(apply dissoc % ks)))
+
+
+(defcache TTLCache [cache ttl ttl-ms]
   CacheProtocol
-  (lookup [_ item]
-    (get cache item))
+  (lookup [this item]
+    (let [ret (lookup this item ::nope)]
+      (when-not (= ret ::nope) ret)))
+  (lookup [this item not-found]
+    (if (has? this item)
+      (get cache item)
+      not-found))
   (has? [_ item]
-    (when-let [t (get ttl item)]
+    (let [t (get ttl item (- ttl-ms))]
       (< (- (System/currentTimeMillis)
             t)
-         limit)))
+         ttl-ms)))
   (hit [this item] this)
   (miss [this item result]
     (let [now  (System/currentTimeMillis)
-          kill-old (key-killer ttl limit now)]
+          kill-old (key-killer ttl ttl-ms now)]
       (TTLCache. (assoc (kill-old cache) item result)
                  (assoc (kill-old ttl) item now)
-                 limit)))
+                 ttl-ms)))
   (seed [_ base]
     (let [now (System/currentTimeMillis)]
       (TTLCache. base
                  (into {} (for [x base] [(key x) now]))
-                 limit)))
+                 ttl-ms)))
   (evict [_ key]
     (TTLCache. (dissoc cache key)
-               (dissoc cache key)
-               limit))
+               (dissoc ttl key)
+               ttl-ms))
   Object
   (toString [_]
-    (str cache \, \space ttl \, \space limit)))
-
-
-(defn- key-killer
-  [ttl limit now]
-  (let [ks (map key (filter #(> (- now (val %)) limit)
-                            ttl))]
-    #(apply dissoc % ks)))
+    (str cache \, \space ttl \, \space ttl-ms)))
 
 
 (defcache LUCache [cache lu limit]
   CacheProtocol
   (lookup [_ item]
     (get cache item))
+  (lookup [_ item not-found]
+    (get cache item not-found))
   (has? [_ item]
     (contains? cache item))
   (hit [_ item]
     (LUCache. cache (update-in lu [item] inc) limit))
   (miss [_ item result]
-    (if-let [ks (keys lu)]
-      (let [k (apply min-key lu ks)]
-        (LUCache. (-> cache (dissoc k) (assoc item result))  ;; expulsion case
-                  (-> lu (dissoc k) (assoc item 0))
-                  limit))
+    (if (>= (count lu) limit) ;; need to evict?
+      (let [min-key (if (contains? lu item)
+                    ::nope
+                    (first (peek lu))) ;; maybe evict case
+            c (-> cache (dissoc min-key) (assoc item result))
+            l (-> lu (dissoc min-key) (update-in [item] (fnil inc 0)))]
+        (LUCache. c l limit))
       (LUCache. (assoc cache item result)  ;; no change case
                 (assoc lu item 0)
                 limit)))
@@ -254,7 +317,7 @@
                    limit))))
   (seed [_ base]
     (LUCache. base
-              (into {} (for [x (range (- limit) 0)] [x x]))
+              (build-leastness-queue base limit 0)
               limit))
   Object
   (toString [_]
@@ -350,6 +413,8 @@
   CacheProtocol
   (lookup [_ item]
     (get cache item))
+  (lookup [_ item not-found]
+    (get cache item not-found))
   (has? [_ item]
     (contains? cache item))
   (hit [_ item]
@@ -416,6 +481,73 @@
   (toString [_]
     (str cache \, \space lruS \, \space lruQ \, \space tick \, \space limitS \, \space limitQ)))
 
+(defn clear-soft-cache! [^java.util.Map cache ^java.util.Map rcache ^ReferenceQueue rq]
+  (loop [r (.poll rq)]
+    (when r
+      (.remove cache (get rcache r))
+      (.remove rcache r)
+      (recur (.poll rq)))))
+
+(defn ^{:dynamic true} make-reference [v rq]
+  (if (nil? v)
+    (SoftReference. ::nil rq)
+    (SoftReference. v rq)))
+
+(defcache SoftCache [^java.util.Map cache ^java.util.Map rcache rq]
+  CacheProtocol
+  (lookup [_ item]
+    (when-let [^SoftReference r (get cache (or item ::nil))]
+      (if (= ::nil (.get r))
+        nil
+        (.get r))))
+  (lookup [_ item not-found]
+    (if-let [^SoftReference r (get cache (or item ::nil))]
+      (if-let [v (.get r)]
+        (if (= ::nil v)
+          nil
+          v)
+        not-found)
+      not-found))
+  (has? [_ item]
+    (let [item (or item ::nil)
+          ^SoftReference cell (get cache item)]
+      (and (contains? cache item)
+           (not (nil? (.get cell))))))
+  (hit [this item]
+    (clear-soft-cache! cache rcache rq)
+    this)
+  (miss [this item result]
+    (let [item (or item ::nil)
+          r (make-reference result rq)]
+      (.put cache item r)
+      (.put rcache r item)
+      (clear-soft-cache! cache rcache rq)
+      this))
+  (evict [this key]
+    (let [key (or key ::nil)
+          r (get cache key)]
+      (when r
+        (.remove cache key)
+        (.remove rcache r))
+      (clear-soft-cache! cache rcache rq)
+      this))
+  (seed [_ base]
+    (let [soft-cache? (instance? SoftCache base)
+          cache (ConcurrentHashMap.)
+          rcache (ConcurrentHashMap.)
+          rq (ReferenceQueue.)]
+      (if (seq base)
+        (doseq [[k ^SoftReference v] base]
+          (let [k (or k ::nil)
+                r (if soft-cache?
+                    (make-reference (.get v) rq)
+                    (make-reference v rq))]
+            (.put cache k r)
+            (.put rcache r k))))
+      (SoftCache. cache rcache rq)))
+  Object
+  (toString [_] (str cache)))
+
 ;; Factories
 
 (defn basic-cache-factory
@@ -431,47 +563,87 @@
    eventual eviction order.  Otherwise, there are no guarantees for the eventual
    eviction ordering.
 
+   This function takes an optional `:threshold` argument that defines the maximum number
+   of elements in the cache before the FIFO semantics apply (default is 32).
+
    If the number of elements in `base` is greater than the limit then some items
    in `base` will be dropped from the resulting cache.  If the associative
    structure used as `base` can guarantee sorting, then the last `limit` elements
    will be used as the cache seed values.  Otherwise, there are no guarantees about
    the elements in the resulting cache."
-  [limit base]
-  {:pre [(number? limit) (< 0 limit)
+  [base & {threshold :threshold :or {threshold 32}}]
+  {:pre [(number? threshold) (< 0 threshold)
          (map? base)]
-   :post [(== limit (count (.q %)))]}
-  (clojure.core.cache/seed (FIFOCache. {} clojure.lang.PersistentQueue/EMPTY limit) base))
+   :post [(== threshold (count (.q ^FIFOCache %)))]}
+  (clojure.core.cache/seed (FIFOCache. {} clojure.lang.PersistentQueue/EMPTY threshold) base))
 
 (defn lru-cache-factory
   "Returns an LRU cache with the cache and usage-table initialied to `base` --
-   each entry is initialized with the same usage value. (maybe this should be
-   randomized?)"
-  [limit base]
-  {:pre [(number? limit) (< 0 limit)
+   each entry is initialized with the same usage value.
+
+   This function takes an optional `:threshold` argument that defines the maximum number
+   of elements in the cache before the LRU semantics apply (default is 32)."
+  [base & {threshold :threshold :or {threshold 32}}]
+  {:pre [(number? threshold) (< 0 threshold)
          (map? base)]}
-  (clojure.core.cache/seed (LRUCache. {} {} 0 limit) base))
+  (clojure.core.cache/seed (LRUCache. {} (clojure.data.priority-map/priority-map) 0 threshold) base))
 
 (defn ttl-cache-factory
   "Returns a TTL cache with the cache and expiration-table initialied to `base` --
-   each with the same time-to-live."
-  [ttl base]
+   each with the same time-to-live.
+
+   This function also allows an optional `:ttl` argument that defines the default
+   time in milliseconds that entries are allowed to reside in the cache."
+  [base & {ttl :ttl :or {ttl 2000}}]
   {:pre [(number? ttl) (<= 0 ttl)
          (map? base)]}
-  (TTLCache. base {} ttl))
+  (clojure.core.cache/seed (TTLCache. {} {} ttl) base))
 
 (defn lu-cache-factory
-  "Returns an LU cache with the cache and usage-table initialied to `base`."
-  [limit base]
-  {:pre [(number? limit) (< 0 limit)
+  "Returns an LU cache with the cache and usage-table initialied to `base`.
+
+   This function takes an optional `:threshold` argument that defines the maximum number
+   of elements in the cache before the LU semantics apply (default is 32)."
+  [base & {threshold :threshold :or {threshold 32}}]
+  {:pre [(number? threshold) (< 0 threshold)
          (map? base)]}
-  (clojure.core.cache/seed (LUCache. {} {} limit) base))
+  (clojure.core.cache/seed (LUCache. {} (clojure.data.priority-map/priority-map) threshold) base))
 
 (defn lirs-cache-factory
   "Returns an LIRS cache with the S & R LRU lists set to the indicated
-   limts."
-  [s-history-limit q-history-limit base]
+   limits."
+  [base & {:keys [s-history-limit q-history-limit]
+           :or {s-history-limit 32
+                q-history-limit 32}}]
   {:pre [(number? s-history-limit) (< 0 s-history-limit)
          (number? q-history-limit) (< 0 q-history-limit)
          (map? base)]}
   (seed (LIRSCache. {} {} {} 0 s-history-limit q-history-limit) base))
 
+(defn soft-cache-factory
+  "Returns a SoftReference cache.  Cached values will be referred to with
+  SoftReferences, allowing the values to be garbage collected when there is
+  memory pressure on the JVM.
+
+  SoftCache is a mutable cache, since it is always based on a
+  ConcurrentHashMap."
+  [base]
+  {:pre [(map? base)]}
+  (seed (SoftCache. (ConcurrentHashMap.) (ConcurrentHashMap.) (ReferenceQueue.))
+        base))
+
+(comment
+
+  (def C (-> {:a 1 :b 2}
+             (fifo-cache-factory :threshold 2)
+             (ttl-cache-factory  :ttl 5000)))
+
+  (assoc C :c 42)
+  ;;=> {:b 2, :c 42}
+
+  ;; wait 5 seconds
+
+  (assoc C :d 138)
+  ;;=> {:d 138}
+
+)
