@@ -1,7 +1,7 @@
 (ns puppetlabs.puppetdb.command-test
   (:require [me.raynes.fs :as fs]
             [clj-http.client :as client]
-            [clojure.java.jdbc.deprecated :as sql]
+            [clojure.java.jdbc :as sql]
             [cheshire.core :as json]
             [puppetlabs.puppetdb.scf.storage :as scf-store]
             [puppetlabs.puppetdb.scf.storage-utils :as sutils]
@@ -12,11 +12,12 @@
             [clj-time.format :as tfmt]
             [puppetlabs.puppetdb.cli.services :as cli-svc]
             [puppetlabs.puppetdb.command :refer :all]
+            [puppetlabs.puppetdb.config :as conf]
+            [puppetlabs.puppetdb.reports :as reports]
             [puppetlabs.puppetdb.testutils :refer :all]
             [puppetlabs.puppetdb.fixtures :refer :all]
             [puppetlabs.puppetdb.jdbc :refer [query-to-vec] :as jdbc]
             [puppetlabs.puppetdb.examples :refer :all]
-            [puppetlabs.puppetdb.testutils.reports :refer [munge-example-report-for-storage]]
             [puppetlabs.puppetdb.testutils.services :as svc-utils]
             [puppetlabs.puppetdb.command.constants :refer [command-names]]
             [clj-time.coerce
@@ -28,7 +29,9 @@
             [puppetlabs.puppetdb.mq-listener :as mql]
             [puppetlabs.puppetdb.utils :as utils]
             [puppetlabs.puppetdb.time :as pt]
-            [puppetlabs.trapperkeeper.app :refer [get-service]])
+            [puppetlabs.trapperkeeper.app :refer [get-service]]
+            [clojure.core.async :as async]
+            [puppetlabs.kitchensink.core :as ks])
   (:import [org.joda.time DateTime DateTimeZone]))
 
 (use-fixtures :each with-test-db)
@@ -238,19 +241,19 @@
 
 (def catalog-versions
   "Currently supported catalog versions"
-  [:v6])
+  [:v7])
 
 (deftest replace-catalog
   (doverseq [version catalog-versions
-             :let [command {:command (command-names :replace-catalog)
-                            :version 6
-                            :payload (-> (get-in wire-catalogs [6 :empty])
+             :let [raw-command {:command (command-names :replace-catalog)
+                            :version 7
+                            :payload (-> (get-in wire-catalogs [7 :empty])
                                          (assoc :producer_timestamp (now)))}]]
     (testing (str (command-names :replace-catalog) " " version)
-      (let [certname (get-in command [:payload :certname])
+      (let [certname (get-in raw-command [:payload :certname])
             catalog-hash (shash/catalog-similarity-hash
-                          (catalog/parse-catalog (:payload command) (version-kwd->num version) (now)))
-            command (stringify-payload command)
+                          (catalog/parse-catalog (:payload raw-command) (version-kwd->num version) (now)))
+            command (stringify-payload raw-command)
             one-day      (* 24 60 60 1000)
             yesterday    (to-timestamp (- (System/currentTimeMillis) one-day))
             tomorrow     (to-timestamp (+ (System/currentTimeMillis) one-day))]
@@ -263,17 +266,28 @@
               (is (= 0 (times-called publish)))
               (is (empty? (fs/list-dir discard-dir))))))
 
+        (testing "with code-id should store the catalog"
+          (with-fixtures
+            (test-msg-handler (-> raw-command
+                                  (assoc-in [:payload :code_id] "my_git_sha1")
+                                  stringify-payload)
+                              publish discard-dir
+                              (is (= [(with-env {:certname certname :code_id "my_git_sha1"})]
+                                     (query-to-vec "SELECT certname, code_id, environment_id FROM catalogs")))
+                              (is (= 0 (times-called publish)))
+                              (is (empty? (fs/list-dir discard-dir))))))
+
         (testing "with an existing catalog should replace the catalog"
           (with-fixtures
             (is (= (query-to-vec "SELECT certname FROM catalogs")
                    []))
-            (sql/insert-record :certnames {:certname certname})
-            (sql/insert-records :catalogs
-                                {:hash (sutils/munge-hash-for-storage "00")
-                                 :api_version 1
-                                 :catalog_version "foo"
-                                 :certname certname
-                                 :producer_timestamp (to-timestamp (-> 1 days ago))})
+            (jdbc/insert! :certnames {:certname certname})
+            (jdbc/insert! :catalogs
+                          {:hash (sutils/munge-hash-for-storage "00")
+                           :api_version 1
+                           :catalog_version "foo"
+                           :certname certname
+                           :producer_timestamp (to-timestamp (-> 1 days ago))})
 
             (test-msg-handler command publish discard-dir
               (is (= [(with-env {:certname certname :catalog catalog-hash})]
@@ -284,16 +298,17 @@
 
         (testing "when replacing a catalog with a debug directory, should write out catalogs for inspection"
           (with-fixtures
-            (sql/insert-record :certnames {:certname certname})
+            (jdbc/insert! :certnames {:certname certname})
 
             (let [debug-dir (fs/absolute-path (temp-dir "catalog-inspection"))]
 
-              (sql/insert-records :catalogs
-                                  {:hash (sutils/munge-hash-for-storage "0000")
-                                   :api_version 1
-                                   :catalog_version "foo"
-                                   :certname certname
-                                   :producer_timestamp (to-timestamp (t/minus (now) (-> 1 days)))})
+              (jdbc/insert! :catalogs
+                            {:hash (sutils/munge-hash-for-storage "0000")
+                             :api_version 1
+                             :catalog_version "foo"
+                             :certname certname
+                             :producer_timestamp (to-timestamp
+                                                  (t/minus (now) (-> 1 days)))})
 
               (is (nil? (fs/list-dir debug-dir)))
               (test-msg-handler-with-opts command publish discard-dir {:catalog-hash-debug-dir debug-dir}
@@ -314,15 +329,15 @@
 
         (testing "with a newer catalog should ignore the message"
           (with-fixtures
-            (sql/insert-record :certnames {:certname certname})
-            (sql/insert-record :catalogs
-                               {:id 1
-                                :hash (sutils/munge-hash-for-storage "ab")
-                                :api_version 1
-                                :catalog_version "foo"
-                                :certname certname
-                                :timestamp tomorrow
-                                :producer_timestamp (to-timestamp (now))})
+            (jdbc/insert! :certnames {:certname certname})
+            (jdbc/insert! :catalogs
+                          {:id 1
+                           :hash (sutils/munge-hash-for-storage "ab")
+                           :api_version 1
+                           :catalog_version "foo"
+                           :certname certname
+                           :timestamp tomorrow
+                           :producer_timestamp (to-timestamp (now))})
 
             (test-msg-handler command publish discard-dir
               (is (= [{:certname certname :catalog "ab"}]
@@ -334,7 +349,7 @@
 
         (testing "should reactivate the node if it was deactivated before the message"
           (with-fixtures
-            (sql/insert-record :certnames {:certname certname :deactivated yesterday})
+            (jdbc/insert! :certnames {:certname certname :deactivated yesterday})
             (test-msg-handler command publish discard-dir
               (is (= [{:certname certname :deactivated nil}]
                      (query-to-vec "SELECT certname,deactivated FROM certnames")))
@@ -346,7 +361,7 @@
 
         (testing "should store the catalog if the node was deactivated after the message"
           (scf-store/delete-certname! certname)
-          (sql/insert-record :certnames {:certname certname :deactivated tomorrow})
+          (jdbc/insert! :certnames {:certname certname :deactivated tomorrow})
           (test-msg-handler command publish discard-dir
             (is (= [{:certname certname :deactivated tomorrow}]
                    (query-to-vec "SELECT certname,deactivated FROM certnames")))
@@ -360,6 +375,29 @@
 ;; potentially have commands of an unsupported format that need to be
 ;; processed. Although we don't support the catalog versions below, we
 ;; need to test that those commands will be processed properly
+(deftest replace-catalog-with-v6
+  (testing "catalog wireformat v6"
+    (let [command {:command (command-names :replace-catalog)
+                   :version 6
+                   :payload (get-in wire-catalogs [6 :empty])}
+          certname (get-in command [:payload :certname])
+          cmd-producer-timestamp (get-in command [:payload :producer_timestamp])]
+      (with-fixtures
+        (test-msg-handler command publish discard-dir
+
+          ;;names in v5 are hyphenated, this check ensures we're sending a v5 catalog
+          (is (contains? (:payload command) :producer_timestamp))
+          (is (= [(with-env {:certname certname})]
+                 (query-to-vec "SELECT certname, environment_id FROM catalogs")))
+          (is (= 0 (times-called publish)))
+          (is (empty? (fs/list-dir discard-dir)))
+
+          ;;this should be the hyphenated producer timestamp provided above
+          (is (= (-> (query-to-vec "SELECT producer_timestamp FROM catalogs")
+                     first
+                     :producer_timestamp)
+                 (to-timestamp cmd-producer-timestamp))))))))
+
 (deftest replace-catalog-with-v5
   (testing "catalog wireformat v5"
     (let [command {:command (command-names :replace-catalog)
@@ -421,12 +459,12 @@
                        resources)))))
 
 (def basic-wire-catalog
-  (get-in wire-catalogs [6 :basic]))
+  (get-in wire-catalogs [7 :basic]))
 
 (deftest catalog-with-updated-resource-line
   (doverseq [version catalog-versions
              :let [command {:command (command-names :replace-catalog)
-                            :version 6
+                            :version 7
                             :payload basic-wire-catalog}
                    command-1 (stringify-payload command)
                    command-2 (stringify-payload (update-resource version command "File" "/etc/foobar" #(assoc % :line 20)))]]
@@ -447,7 +485,7 @@
 (deftest catalog-with-updated-resource-file
   (doverseq [version catalog-versions
              :let [command {:command (command-names :replace-catalog)
-                            :version 6
+                            :version 7
                             :payload basic-wire-catalog}
                    command-1 (stringify-payload command)
                    command-2 (stringify-payload (update-resource version command "File" "/etc/foobar" #(assoc % :file "/tmp/not-foo")))]]
@@ -467,7 +505,7 @@
 (deftest catalog-with-updated-resource-exported
   (doverseq [version catalog-versions
              :let [command {:command (command-names :replace-catalog)
-                            :version 6
+                            :version 7
                             :payload basic-wire-catalog}
                    command-1 (stringify-payload command)
                    command-2 (stringify-payload (update-resource version command "File" "/etc/foobar" #(assoc % :exported true)))]]
@@ -488,7 +526,7 @@
 (deftest catalog-with-updated-resource-tags
   (doverseq [version catalog-versions
              :let [command {:command (command-names :replace-catalog)
-                            :version 6
+                            :version 7
                             :payload basic-wire-catalog}
                    command-1 (stringify-payload command)
                    command-2 (stringify-payload
@@ -571,7 +609,7 @@
     (doverseq [version fact-versions
                :let [command v4-command]]
 
-      (sql/transaction
+      (jdbc/with-db-transaction []
        (scf-store/ensure-environment "DEV")
        (scf-store/add-certname! certname)
        (scf-store/replace-facts! {:certname certname
@@ -613,7 +651,7 @@
     (doverseq [version fact-versions
                :let [command v4-command]]
 
-      (sql/transaction
+      (jdbc/with-db-transaction []
        (scf-store/ensure-environment "DEV")
        (scf-store/add-certname! certname)
        (scf-store/add-facts! {:certname certname
@@ -651,7 +689,7 @@
                :let [command v4-command]]
 
       (testing "should reactivate the node if it was deactivated before the message"
-        (sql/insert-record :certnames {:certname certname :deactivated yesterday})
+        (jdbc/insert! :certnames {:certname certname :deactivated yesterday})
         (test-msg-handler command publish discard-dir
           (is (= (query-to-vec "SELECT certname,deactivated FROM certnames")
                  [{:certname certname :deactivated nil}]))
@@ -677,7 +715,7 @@
 
       (testing "should store the facts if the node was deactivated after the message"
         (scf-store/delete-certname! certname)
-        (sql/insert-record :certnames {:certname certname :deactivated tomorrow})
+        (jdbc/insert! :certnames {:certname certname :deactivated tomorrow})
         (test-msg-handler command publish discard-dir
           (is (= (query-to-vec "SELECT certname,deactivated FROM certnames")
                  [{:certname certname :deactivated tomorrow}]))
@@ -746,7 +784,7 @@
 
 (deftest replace-facts-with-v2-wire-format
   (let [certname  "foo.example.com"
-        before-test-starts-time (now)
+        before-test-starts-time (-> 1 seconds ago)
         facts-cmd {:command (command-names :replace-facts)
                    :version 2
                    :payload {:name certname
@@ -775,12 +813,12 @@
               {:certname certname :name "b" :value "2" :environment "DEV"}
               {:certname certname :name "c" :value "3" :environment "DEV"}]))
 
-      (is (= (every? (comp #(t/before? before-test-starts-time %)
-                           to-date-time
-                           :producer_timestamp)
-                     (query-to-vec
-                      "SELECT fs.producer_timestamp
-                         FROM factsets fs"))))
+      (is (every? (comp #(t/before? before-test-starts-time %)
+                        to-date-time
+                        :producer_timestamp)
+                  (query-to-vec
+                   "SELECT fs.producer_timestamp
+                         FROM factsets fs")))
       (is (= 0 (times-called publish)))
       (is (empty? (fs/list-dir discard-dir)))
       (let [result (query-to-vec "SELECT certname,environment_id FROM factsets")]
@@ -830,7 +868,7 @@
           hand-off-queue (java.util.concurrent.SynchronousQueue.)
           storage-replace-facts! scf-store/update-facts!]
 
-      (sql/transaction
+      (jdbc/with-db-transaction []
        (scf-store/add-certname! certname)
        (scf-store/add-facts! {:certname certname
                               :values (:values facts)
@@ -951,7 +989,7 @@
         storage-delete-pending-path-id-orphans!
         scf-store/delete-pending-path-id-orphans!]
 
-    (sql/transaction
+    (jdbc/with-db-transaction []
      (scf-store/add-certname! certname-1)
      (scf-store/add-certname! certname-2)
      (scf-store/add-facts! {:certname certname-1
@@ -1032,7 +1070,7 @@
           hand-off-queue (java.util.concurrent.SynchronousQueue.)
           storage-replace-catalog! scf-store/replace-catalog!]
 
-      (sql/transaction
+      (jdbc/with-db-transaction []
        (scf-store/add-certname! certname)
        (scf-store/replace-catalog! nonwire-catalog (-> 2 days ago)))
 
@@ -1077,7 +1115,7 @@
           hand-off-queue (java.util.concurrent.SynchronousQueue.)
           storage-replace-catalog! scf-store/replace-catalog!]
 
-      (sql/transaction
+      (jdbc/with-db-transaction []
        (scf-store/add-certname! certname)
        (scf-store/replace-catalog! nonwire-catalog (-> 2 days ago)))
 
@@ -1139,7 +1177,7 @@
   (deftest deactivate-node-node-active
     (testing "should deactivate the node"
       (doseq [{:keys [certname command]} cases]
-        (sql/insert-record :certnames {:certname certname})
+        (jdbc/insert! :certnames {:certname certname})
         (test-msg-handler command publish discard-dir
           (let [results (query-to-vec "SELECT certname,deactivated FROM certnames")
                 result  (first results)]
@@ -1147,7 +1185,7 @@
             (is (instance? java.sql.Timestamp (:deactivated result)))
             (is (= 0 (times-called publish)))
             (is (empty? (fs/list-dir discard-dir)))
-            (sql/do-prepared "delete from certnames"))))))
+            (jdbc/do-prepared "delete from certnames"))))))
 
   (deftest deactivate-node-node-inactive
     (doseq [{:keys [certname command]} cases]
@@ -1161,7 +1199,8 @@
                        command
                        (assoc-in command
                                  [:payload :producer_timestamp] yesterday))]
-         (sql/insert-record :certnames {:certname certname :deactivated yesterday})
+         (jdbc/insert! :certnames
+                       {:certname certname :deactivated yesterday})
          (test-msg-handler
            command
            publish discard-dir
@@ -1178,7 +1217,7 @@
                (is (= {:certname certname :deactivated yesterday} row)))
              (is (= 0 (times-called publish)))
              (is (empty? (fs/list-dir discard-dir)))
-             (sql/do-prepared "delete from certnames")))))))
+             (jdbc/do-prepared "delete from certnames")))))))
 
   (deftest deactivate-node-node-missing
     (testing "should add the node and deactivate it"
@@ -1186,11 +1225,11 @@
         (test-msg-handler command publish discard-dir
           (let [results (query-to-vec "SELECT certname,deactivated FROM certnames")
                 result  (first results)]
-            (is (= (:certname result) certname ))
+            (is (= (:certname result) certname))
             (is (instance? java.sql.Timestamp (:deactivated result)))
-            (is (= 0 (times-called publish)))
+            (is (zero? (times-called publish)))
             (is (empty? (fs/list-dir discard-dir)))
-            (sql/do-prepared "delete from certnames")))))))
+            (jdbc/do-prepared "delete from certnames")))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
@@ -1198,114 +1237,98 @@
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(def report-versions
-  "Report versions supported. Version 1 is not currently being tested."
-  [:v5])
+(def v5-report
+  (-> (:basic report-examples/reports)
+      (assoc :environment "DEV")
+      reports/report-query->wire-v5))
 
-(defn- resource-event->resource
-  [resource-event]
-  (-> resource-event
-      (select-keys [:file :line :timestamp :resource_type :resource_title :containment_path])
-      (assoc :skipped (= "skipped" (:status resource-event)))))
+(def v4-report
+  (-> v5-report
+      (dissoc :producer_timestamp :metrics :logs :noop)
+      utils/underscore->dash-keys))
 
-(defn- resource-events->resources
-  [resource-events]
-  (for [[resource events] (group-by resource-event->resource resource-events)]
-    (assoc resource :events (map #(dissoc % :file :line :resource_type :resource_title :containment_path) events))))
+(def store-report (command-names :store-report))
 
-(let [report (-> (:basic report-examples/reports)
-                 (assoc :environment "DEV")
-                 munge-example-report-for-storage
-                 (update :resource_events resource-events->resources)
-                 (clojure.set/rename-keys {:resource_events :resources}))
-      v6-command {:command (command-names :store-report)
-                  :version 6
-                  :payload report}]
-  (deftest store-v6-report
-    (testing "should store the report"
-      (test-msg-handler v6-command publish discard-dir
-        (is (= [(with-env (select-keys report [:certname :configuration_version]))]
-               (query-to-vec "SELECT certname,configuration_version,environment_id FROM reports")))
-        (is (= 0 (times-called publish)))
-        (is (empty? (fs/list-dir discard-dir)))))))
+(deftest store-v6-report-test
+  (let [v6-report (-> v5-report
+                      (update :resource_events reports/resource-events-v5->resources)
+                      (clojure.set/rename-keys {:resource_events :resources}))
+        command {:command store-report
+                 :version 6
+                 :payload v6-report}]
+    (test-msg-handler command publish discard-dir
+      (is (= (query-to-vec "SELECT certname,configuration_version,environment_id FROM reports")
+             [(with-env (select-keys v6-report [:certname :configuration_version]))]))
+      (is (= 0 (times-called publish)))
+      (is (empty? (fs/list-dir discard-dir))))))
 
-(let [report (-> (:basic report-examples/reports)
-                 (assoc :environment "DEV")
-                 munge-example-report-for-storage)
-      v5-command {:command (command-names :store-report)
-                  :version 5
-                  :payload report}]
-  (deftest store-report
-    (testing "should store the report"
-      (doverseq [version report-versions
-                 :let [command v5-command]]
-        (test-msg-handler command publish discard-dir
-          (is (= (query-to-vec "SELECT certname,configuration_version,environment_id FROM reports")
-                 [(with-env {:certname (:certname report) :configuration_version (:configuration_version report)})]))
-          (is (= 0 (times-called publish)))
-          (is (empty? (fs/list-dir discard-dir))))))))
+(deftest store-v5-report-test
+  (let [command {:command store-report
+                 :version 5
+                 :payload v5-report}]
+    (test-msg-handler command publish discard-dir
+      (is (= (query-to-vec "SELECT certname,configuration_version,environment_id FROM reports")
+             [(with-env (select-keys v5-report [:certname :configuration_version]))]))
+      (is (= 0 (times-called publish)))
+      (is (empty? (fs/list-dir discard-dir))))))
 
-(let [old-report (-> (:basic report-examples/reports)
-                     (assoc :environment "DEV")
-                     munge-example-report-for-storage
-                     (dissoc :producer_timestamp :metrics :logs :noop)
-                     utils/underscore->dash-keys)
-      recent-time (-> 1 seconds ago)]
-  (deftest store-v4-report
-    (let [command {:command (command-names :store-report)
-                   :version 4
-                   :payload old-report}]
-      (test-msg-handler command publish discard-dir
-        (is (= (query-to-vec "SELECT certname,configuration_version,environment_id FROM reports")
-               [(with-env {:certname (:certname old-report)
-                           :configuration_version (:configuration-version old-report)})]))
+(deftest store-v4-report-test
+  (let [command {:command store-report
+                 :version 4
+                 :payload v4-report}
+        recent-time (-> 1 seconds ago)]
+    (test-msg-handler command publish discard-dir
+      (is (= [(with-env (utils/dash->underscore-keys
+                         (select-keys v4-report [:certname :configuration-version])))]
+             (query-to-vec "SELECT certname,configuration_version,environment_id FROM reports")))
 
-        ;;Status is present in v4+ (but not in v3)
-        (is (= "unchanged" (-> (query-to-vec "SELECT rs.status FROM reports r inner join report_statuses rs on r.status_id = rs.id")
-                               first
-                               :status)))
+      ;;Status is present in v4+ (but not in v3)
+      (is (= "unchanged" (-> (query-to-vec "SELECT rs.status FROM reports r inner join report_statuses rs on r.status_id = rs.id")
+                             first
+                             :status)))
 
-        ;;No producer_timestamp is included in v4, message received time (now) is used intead
-        (is (t/before? recent-time (-> (query-to-vec "SELECT producer_timestamp FROM reports")
-                                        first
-                                        :producer_timestamp
-                                        to-date-time)))
-        (is (= 0 (times-called publish)))
-        (is (empty? (fs/list-dir discard-dir))))))
+      ;;No producer_timestamp is included in v4, message received time (now) is used intead
+      (is (t/before? recent-time (-> (query-to-vec "SELECT producer_timestamp FROM reports")
+                                     first
+                                     :producer_timestamp
+                                     to-date-time)))
+      (is (= 0 (times-called publish)))
+      (is (empty? (fs/list-dir discard-dir))))))
 
-  (deftest store-v3-report
-    (let [recent-time (-> 1 seconds ago)
-          command {:command (command-names :store-report)
-                   :version 3
-                   :payload (dissoc old-report :status)}]
-      (Thread/sleep 100)
-      (test-msg-handler command publish discard-dir
-        (is (= (query-to-vec "SELECT certname,configuration_version,environment_id FROM reports")
-               [(with-env {:certname (:certname old-report)
-                           :configuration_version (:configuration-version old-report)})]))
+(deftest store-v3-report-test
+  (let [v3-report (dissoc v4-report :status)
+        recent-time (-> 1 seconds ago)
+        command {:command store-report
+                 :version 3
+                 :payload v3-report}]
+    (test-msg-handler command publish discard-dir
+      (is (= [(with-env (utils/dash->underscore-keys
+                         (select-keys v3-report [:certname :configuration-version])))]
+             (query-to-vec "SELECT certname,configuration_version,environment_id FROM reports")))
 
-        ;;No producer_timestamp is included in v4, message received time (now) is used intead
-        (is (t/before? recent-time (-> (query-to-vec "SELECT producer_timestamp FROM reports")
-                                        first
-                                        :producer_timestamp
-                                        to-date-time)))
+      ;;No producer_timestamp is included in v4, message received time (now) is used intead
+      (is (t/before? recent-time (-> (query-to-vec "SELECT producer_timestamp FROM reports")
+                                     first
+                                     :producer_timestamp
+                                     to-date-time)))
 
-        ;;Status is not supported in v3, should be nil
-        (is (nil? (-> (query-to-vec "SELECT status_id FROM reports")
-                      first
-                      :status)))
-        (is (= 0 (times-called publish)))
-        (is (empty? (fs/list-dir discard-dir)))))))
+      ;;Status is not supported in v3, should be nil
+      (is (nil? (-> (query-to-vec "SELECT status_id FROM reports")
+                    first
+                    :status)))
+      (is (= 0 (times-called publish)))
+      (is (empty? (fs/list-dir discard-dir))))))
+
+(defn- get-config []
+  (conf/get-config (get-service svc-utils/*server* :DefaultedConfig)))
 
 (deftest command-service-stats
   (svc-utils/with-puppetdb-instance
     (let [pdb (get-service svc-utils/*server* :PuppetDBServer)
           dispatcher (get-service svc-utils/*server* :PuppetDBCommandDispatcher)
-          shared-globals (partial cli-svc/shared-globals pdb)
           enqueue-command (partial enqueue-command dispatcher)
           stats (partial stats dispatcher)
-          real-replace! scf-store/replace-facts!
-          {{:keys [connection endpoint]} :command-mq} (shared-globals)]
+          real-replace! scf-store/replace-facts!]
       ;; Issue a single command and ensure the stats are right at each step.
       (is (= {:received-commands 0 :executed-commands 0} (stats)))
       (let [received-cmd? (promise)
@@ -1315,8 +1338,7 @@
                         (deliver received-cmd? true)
                         @go-ahead-and-execute
                         (apply real-replace! args))]
-          (enqueue-command connection endpoint
-                           (command-names :replace-facts) 4
+          (enqueue-command (command-names :replace-facts) 4
                            {:environment "DEV" :certname "foo.local"
                             :values {:foo "foo"}
                             :producer_timestamp (to-string (now))})
@@ -1331,23 +1353,19 @@
   (svc-utils/with-puppetdb-instance
     (let [pdb (get-service svc-utils/*server* :PuppetDBServer)
           dispatcher (get-service svc-utils/*server* :PuppetDBCommandDispatcher)
-          shared-globals (partial cli-svc/shared-globals pdb)
           enqueue-command (partial enqueue-command dispatcher)
-          globals (shared-globals)
-          {{:keys [connection endpoint]} :command-mq} globals
           deactivate-ms 14250331086887
           ;; The problem only occurred if you passed a Date to
           ;; enqueue, a DateTime wasn't a problem.
           input-stamp (java.util.Date. deactivate-ms)
           expected-stamp (DateTime. deactivate-ms DateTimeZone/UTC)]
-      (enqueue-command connection endpoint
-                       (command-names :deactivate-node) 3
+      (enqueue-command (command-names :deactivate-node) 3
                        {:certname "foo.local" :producer_timestamp input-stamp})
       (is (svc-utils/wait-for-server-processing svc-utils/*server* 5000))
       ;; While we're here, check the value in the database too...
       (is (= expected-stamp
              (jdbc/with-transacted-connection
-               (:scf-read-db globals)
+               (:scf-read-db (cli-svc/shared-globals pdb))
                :repeatable-read
                (from-sql-date (scf-store/node-deactivated-time "foo.local")))))
       (is (= expected-stamp
@@ -1366,6 +1384,22 @@
                  (get "deactivated")
                  (pt/from-string)))))))
 
+(deftest command-response-channel
+  (svc-utils/with-puppetdb-instance
+    (let [pdb (get-service svc-utils/*server* :PuppetDBServer)
+          dispatcher (get-service svc-utils/*server* :PuppetDBCommandDispatcher)
+          enqueue-command (partial enqueue-command dispatcher)
+          response-mult (response-mult dispatcher)
+          response-chan (async/chan)
+          command-uuid (ks/uuid)]
+      (async/tap response-mult response-chan)
+      (enqueue-command (command-names :deactivate-node) 3
+                       {:certname "foo.local" :producer_timestamp (java.util.Date.)}
+                       command-uuid)
+      (let [received-uuid (async/alt!! response-chan ([msg] (:id msg))
+                                       (async/timeout 2000) ::timeout)]
+       (is (= command-uuid received-uuid))))))
+
 ;; Local Variables:
 ;; mode: clojure
 ;; eval: (define-clojure-indent (test-msg-handler (quote defun))
