@@ -233,3 +233,23 @@
                     (binding [*err* *out*]
                       (warn-retirements bad-config)))]
       (is (.contains out-str "[repl] is now retired")))))
+
+(deftest test-default-max-command-size
+  (testing "default is disabled"
+    (let [default-config (:command-processing (configure-command-processing {}))]
+      (is (false? (:reject-large-commands default-config)))
+      (is (= (long (default-max-command-size))
+             (:max-command-size default-config)))))
+
+  (testing "enabled but with default size"
+    (let [default-config (:command-processing (configure-command-processing {:command-processing {:reject-large-commands "true"}}))]
+      (is (true? (:reject-large-commands default-config)))
+      (is (= (default-max-command-size)
+             (:max-command-size default-config)))))
+
+  (testing "enabled with included max size"
+    (let [default-config (:command-processing (configure-command-processing {:command-processing {:reject-large-commands "true"
+                                                                                                  :max-command-size 25000}}))]
+      (is (true? (:reject-large-commands default-config)))
+      (is (= 25000
+             (:max-command-size default-config))))))
