@@ -1,7 +1,10 @@
 (ns puppetlabs.puppetdb.testutils.db
-  (:require [clojure.java.jdbc.deprecated :as sql]
+  (:require [clojure.java.jdbc :as sql]
+            [puppetlabs.puppetdb.jdbc :as jdbc]
             [puppetlabs.puppetdb.testutils :refer [clear-db-for-testing! test-db]]))
 
+(def ^:dynamic *db-spec* nil)
+
 (def antonym-data {"absence"    "presence"
                    "abundant"   "scarce"
                    "accept"     "refuse"
@@ -20,20 +23,19 @@
                    "blandness"  "zest"
                    "lethargy"   "zest"})
 
-(def ^:dynamic *db-spec* nil)
-
 (defn insert-map [data]
-  (apply (partial sql/insert-values :test [:key :value]) data))
+  (apply (partial jdbc/insert! :test [:key :value]) data))
 
 (defn with-antonym-test-database
   [function]
-  (sql/with-connection (test-db)
-    (clear-db-for-testing!))
-  (binding [*db-spec* (test-db)]
-    (sql/with-connection *db-spec*
-      (sql/transaction
-       (sql/create-table :test
-                         [:key   "VARCHAR(256)" "PRIMARY KEY"]
-                         [:value "VARCHAR(256)" "NOT NULL"])
-       (insert-map antonym-data))
-      (function))))
+  (let [db (test-db)]
+    (binding [*db-spec* db]
+      (jdbc/with-db-connection db
+        (clear-db-for-testing!)
+        (jdbc/with-db-transaction []
+          (jdbc/do-commands
+           (sql/create-table-ddl :test
+                                 [:key "VARCHAR(256)" "PRIMARY KEY"]
+                                 [:value "VARCHAR(256)" "NOT NULL"]))
+          (insert-map antonym-data))
+        (function)))))
